jest.mock("@firstfleet/ffslack", () => ({
    PostToSlack: jest.fn()
}));

jest.mock("@firstfleet/fflogger", () => ({
    error: jest.fn()
}));

const handlers = require("../handlers");
const { slackChannels } = require("../constants");
const slack = require("@firstfleet/ffslack");
const logger = require("@firstfleet/fflogger");

const appName = process.env.PAPERTRAIL_PROGRAM;
const testSender = appName;
const testMethod = "testMethod";
const testDetails = "testDetails";
const testMessage = "testMessage";
const testSeverity = "testSeverity";
const testIsOperational = true;
const testChannelId = slackChannels.WEB;
let testSendToSlack = true;

const handlersCopy = Object.assign({}, handlers);

afterEach(() => {
    process.env.PAPERTRAIL_PROGRAM = "appName";
    jest.resetAllMocks();
})

describe("handlers.js", () => {
    describe("handlerError", () => {

        it("Should call logAndNotify with a channelId if sendToSlack === true", () => {
            handlers.logAndNotify = jest.fn();

            handlers.handleError(testSender, testMethod, testMessage, testSeverity, testIsOperational, testChannelId, testSendToSlack);

            expect(handlers.logAndNotify).toHaveBeenCalledWith(testSender, testMethod, testMessage, "", testChannelId);
        });

        it("Should call logAndNotify without a channelId if sendToSlack = false", () => {
            handlers.logAndNotify = jest.fn();
            testSendToSlack = false;
            handlers.handleError(testSender, testMethod, testMessage, testSeverity, testIsOperational, testChannelId, testSendToSlack);

            expect(handlers.logAndNotify).toHaveBeenCalledWith(testSender, testMethod, testMessage, "");
        });

    });

    describe("logAndNotify", () => {

        it("Should change appName to process.env.PAPERTRAIL_PROGRAM program if blank", () => {
            const differentAppName = "";

            handlers.logAndNotify = handlersCopy.logAndNotify;
            handlers.logAndNotify(differentAppName, testMethod, testMessage, testDetails, testChannelId);

            expect(logger.error).toHaveBeenCalledWith(appName, testMethod, `Error: ${testMessage}\n${testDetails}`, '');
            expect(slack.PostToSlack).toHaveBeenCalledWith(appName, `${testMethod}-${testMessage}`, testChannelId, testDetails);
        });

        it("Should use the passed in appName if PAPERTRAIL_PROGRAM is not in the env variables", () => {
            delete process.env.PAPERTRAIL_PROGRAM;
            const differentAppName = "differentAppName";

            handlers.logAndNotify = handlersCopy.logAndNotify;
            handlers.logAndNotify(differentAppName, testMethod, testMessage, testDetails, testChannelId);

            expect(logger.error).toHaveBeenCalledWith(differentAppName, testMethod, `Error: ${testMessage}\n${testDetails}`, '');
            expect(slack.PostToSlack).toHaveBeenCalledWith(differentAppName, `${testMethod}-${testMessage}`, testChannelId, testDetails);
        });

        it("Should call logger.error", () => {
            handlers.logAndNotify = handlersCopy.logAndNotify;
            handlers.logAndNotify(appName, testMethod, testMessage, testDetails, testChannelId);

            expect(logger.error).toHaveBeenCalledWith(appName, testMethod, `Error: ${testMessage}\n${testDetails}`, '');
        });

        it("Should call PostToSlack if a slackChannel is provided", () => {
            handlers.logAndNotify = handlersCopy.logAndNotify;
            handlers.logAndNotify(appName, testMethod, testMessage, testDetails, testChannelId);

            expect(slack.PostToSlack).toHaveBeenCalledWith(appName, `${testMethod}-${testMessage}`, testChannelId, testDetails);
        });

        it("Should not call PostToSlack if no slackChannel is provided", () => {
            handlers.logAndNotify = handlersCopy.logAndNotify;
            handlers.logAndNotify(appName, testMethod, testMessage, testDetails);

            expect(slack.PostToSlack).not.toHaveBeenCalled();
        });

        it("Should call console.error if it fails to log", () => {
            const consoleErrorSpy = jest.spyOn(console, 'error').mockImplementation()
            const slackError = new Error("test");
            slack.PostToSlack.mockImplementation(() => { throw slackError });

            handlers.logAndNotify = handlersCopy.logAndNotify;
            handlers.logAndNotify(appName, testMethod, testMessage, testDetails, testChannelId);

            expect(consoleErrorSpy).toHaveBeenCalledWith(slackError);

            consoleErrorSpy.mockClear()
            slack.PostToSlack.mockReset();
        });

    });

    describe("PostToSlack", () => {

        it("Should call logAndNotify, and pass through parameters", () => {
            handlers.logAndNotify = jest.fn();
            handlers.PostToSlack(appName, testMessage, testChannelId, testDetails);

            expect(handlers.logAndNotify).toHaveBeenCalledWith(appName, "", testMessage, testDetails, testChannelId);
        });
    })
});
